Imports Microsoft.VisualBasic
Imports System
Imports System.Web.UI
Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.WebEdition
Imports System.Collections.ObjectModel

Partial Public Class DrawEditShapes : Inherits System.Web.UI.Page

    Dim cbOverlay As LayerOverlay
    Dim cbLayer As InMemoryFeatureLayer

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load
        If (Not Page.IsPostBack) Then
            Map1.MapBackground.BackgroundBrush = New GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"))
            Map1.CurrentExtent = New RectangleShape(-131.22, 55.05, -54.03, 16.91)
            Map1.MapUnit = GeographyUnit.DecimalDegree

            Dim worldMapKitWmsWebOverlay As New WorldMapKitWmsWebOverlay()
            Map1.CustomOverlays.Add(worldMapKitWmsWebOverlay)

            Dim shapeLayer As New InMemoryFeatureLayer()
            shapeLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimpleCircleStyle(GeoColor.FromArgb(180, 102, 255, 102), 10, GeoColor.StandardColors.DarkGreen, 1)
            shapeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.StandardColors.Green, 4, True)
            shapeLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(180, 102, 255, 102), GeoColor.StandardColors.DarkGreen, 1)
            shapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
            shapeLayer.DrawingQuality = DrawingQuality.HighQuality

            Dim dynamicOverlay As New LayerOverlay("cbOverlay")
            dynamicOverlay.IsBaseOverlay = False
            dynamicOverlay.Layers.Add("cbLayer", shapeLayer)
            Map1.CustomOverlays.Add(dynamicOverlay)
        End If
    End Sub

    '******************************************************************************

    Private Sub SelectClickedFeature(ByVal ClickedPoint As PointShape)

        'TODO: Warn that unsaved changes to edit layer will be lost


        'Search CB Layer first
        Dim searchPoint As New PointShape(ClickedPoint.X, ClickedPoint.Y)

        cbOverlay = Map1.CustomOverlays("cbOverlay")
        cbLayer = DirectCast(cbOverlay.Layers("cbLayer"), InMemoryFeatureLayer)

        If cbLayer.InternalFeatures.Count > 0 Then
            cbLayer.FeatureSource.Open()
            Dim cbFeaturesClicked As Collection(Of Feature) = _
                cbLayer.QueryTools.GetFeaturesContaining(ClickedPoint, ReturningColumnsType.AllColumns)
            If cbFeaturesClicked.Count > 0 Then

                Map1.EditOverlay.Features.Clear()
                Map1.EditOverlay.Features.Add(cbFeaturesClicked(0))

                cbLayer.InternalFeatures.Remove(cbFeaturesClicked(0))
                cbOverlay.Redraw()

                'TODO: handle multiple shapes clicked

            Else

                If Map1.EditOverlay.Features.Count > 0 Then
                    TurnOnEditing()
                End If

            End If
            cbLayer.FeatureSource.Close()

            If Map1.EditOverlay.Features.Count > 0 Then
                TurnOnEditing()
            End If
        End If


    End Sub

    Private Sub Map1_TrackShapeFinished(ByVal sender As Object, ByVal e As System.EventArgs) Handles Map1.TrackShapeFinished
        cbOverlay = Map1.CustomOverlays("cbOverlay")
        cbLayer = DirectCast(cbOverlay.Layers("cbLayer"), InMemoryFeatureLayer)

        If Map1.EditOverlay.Features.Count > 1 Then

            Dim lastFeature As Feature = Map1.EditOverlay.Features(Map1.EditOverlay.Features.Count - 1)
            Map1.EditOverlay.Features.Clear()
            Map1.EditOverlay.Features.Add(lastFeature)

        End If

        TurnOnEditing()

    End Sub

    Private Sub TurnOnEditing()
        Map1.EditOverlay.EditSettings.IsReshapable = True
        Map1.EditOverlay.EditSettings.IsDraggable = True
        Map1.EditOverlay.EditSettings.IsResizable = True
        Map1.EditOverlay.EditSettings.IsRotatable = False
        Map1.EditOverlay.TrackMode = TrackMode.Edit
        'Map1.EditOverlay.EditTools.BeginTransaction()
    End Sub
    Private Sub TurnOffEditing()
        Map1.EditOverlay.EditSettings.IsReshapable = False
        Map1.EditOverlay.EditSettings.IsDraggable = False
        Map1.EditOverlay.EditSettings.IsResizable = False
        Map1.EditOverlay.EditSettings.IsRotatable = False
        Map1.EditOverlay.TrackMode = TrackMode.None
        'Map1.EditOverlay.EditTools.BeginTransaction()
    End Sub

    Private Sub Map1_Click(ByVal sender As Object, ByVal e As ThinkGeo.MapSuite.WebEdition.MapClickedEventArgs) Handles Map1.Click
        SelectClickedFeature(e.Position)
    End Sub


    Protected Sub buttonSubmit1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSubmit1.Click
        Dim dynamicOverlay As LayerOverlay = Map1.CustomOverlays("cbOverlay")
        Dim shapeLayer As InMemoryFeatureLayer = DirectCast(dynamicOverlay.Layers("cbLayer"), InMemoryFeatureLayer)

        For Each Feature As Feature In Map1.EditOverlay.Features
            shapeLayer.InternalFeatures.Add(Feature.Id, Feature)
        Next

        Map1.EditOverlay.Features.Clear()
        dynamicOverlay.Redraw()
    End Sub

    Protected Sub buttonEditShape1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonEditShape1.Click
        Dim dynamicOverlay As LayerOverlay = Map1.CustomOverlays("cbOverlay")
        Dim shapeLayer As InMemoryFeatureLayer = DirectCast(dynamicOverlay.Layers("cbLayer"), InMemoryFeatureLayer)

        For Each Feature As Feature In shapeLayer.InternalFeatures
            Map1.EditOverlay.Features.Add(Feature.Id, Feature)
        Next

        shapeLayer.InternalFeatures.Clear()
        dynamicOverlay.Redraw()
        Map1.EditOverlay.TrackMode = TrackMode.Edit
    End Sub
End Class
